@@ -29,9 +29,9 @@
 namespace redis {
 
 struct CommandFunction : Commander {
-  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+  Status Execute(engine::Context &ctx, Server *srv, Connection *conn, std::string *output) override {
     CommandParser parser(args_, 1);
-    engine::Context ctx(srv->storage);
+
     if (parser.EatEqICase("load")) {
       bool replace = false;
       if (parser.EatEqICase("replace")) {
@@ -75,7 +75,7 @@ struct CommandFunction : Commander {
       auto s = lua::FunctionDelete(ctx, srv, libname);
       if (!s) return s;
 
-      *output = SimpleString("OK");
+      *output = RESP_OK;
       return Status::OK();
     } else {
       return {Status::NotOK, "no such subcommand"};
@@ -85,7 +85,8 @@ struct CommandFunction : Commander {
 
 template <bool read_only = false>
 struct CommandFCall : Commander {
-  Status Execute([[maybe_unused]] Server *srv, Connection *conn, std::string *output) override {
+  Status Execute([[maybe_unused]] engine::Context &ctx, [[maybe_unused]] Server *srv, Connection *conn,
+                 std::string *output) override {
     int64_t numkeys = GET_OR_RET(ParseInt<int64_t>(args_[2], 10));
     if (numkeys > int64_t(args_.size() - 3)) {
       return {Status::NotOK, "Number of keys can't be greater than number of args"};
@@ -108,9 +109,10 @@ uint64_t GenerateFunctionFlags(uint64_t flags, const std::vector<std::string> &a
   return flags;
 }
 
-REDIS_REGISTER_COMMANDS(
-    Function, MakeCmdAttr<CommandFunction>("function", -2, "exclusive no-script", 0, 0, 0, GenerateFunctionFlags),
-    MakeCmdAttr<CommandFCall<>>("fcall", -3, "exclusive write no-script", GetScriptEvalKeyRange),
-    MakeCmdAttr<CommandFCall<true>>("fcall_ro", -3, "read-only ro-script no-script", GetScriptEvalKeyRange));
+REDIS_REGISTER_COMMANDS(Function,
+                        MakeCmdAttr<CommandFunction>("function", -2, "exclusive no-script", NO_KEY,
+                                                     GenerateFunctionFlags),
+                        MakeCmdAttr<CommandFCall<>>("fcall", -3, "exclusive write no-script", GetScriptEvalKeyRange),
+                        MakeCmdAttr<CommandFCall<true>>("fcall_ro", -3, "read-only no-script", GetScriptEvalKeyRange));
 
 }  // namespace redis
