@@ -29,10 +29,10 @@ namespace redis {
 
 class CommandHGet : public Commander {
  public:
-  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+  Status Execute(engine::Context &ctx, Server *srv, Connection *conn, std::string *output) override {
     redis::Hash hash_db(srv->storage, conn->GetNamespace());
     std::string value;
-    engine::Context ctx(srv->storage);
+
     auto s = hash_db.Get(ctx, args_[1], args_[2], &value);
     if (!s.ok() && !s.IsNotFound()) {
       return {Status::RedisExecErr, s.ToString()};
@@ -55,10 +55,10 @@ class CommandHSetNX : public Commander {
     return Commander::Parse(args);
   }
 
-  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+  Status Execute(engine::Context &ctx, Server *srv, Connection *conn, std::string *output) override {
     uint64_t ret = 0;
     redis::Hash hash_db(srv->storage, conn->GetNamespace());
-    engine::Context ctx(srv->storage);
+
     auto s = hash_db.MSet(ctx, args_[1], field_values_, true, &ret);
     if (!s.ok()) {
       return {Status::RedisExecErr, s.ToString()};
@@ -74,10 +74,10 @@ class CommandHSetNX : public Commander {
 
 class CommandHStrlen : public Commander {
  public:
-  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+  Status Execute(engine::Context &ctx, Server *srv, Connection *conn, std::string *output) override {
     redis::Hash hash_db(srv->storage, conn->GetNamespace());
     std::string value;
-    engine::Context ctx(srv->storage);
+
     auto s = hash_db.Get(ctx, args_[1], args_[2], &value);
     if (!s.ok() && !s.IsNotFound()) {
       return {Status::RedisExecErr, s.ToString()};
@@ -90,7 +90,7 @@ class CommandHStrlen : public Commander {
 
 class CommandHDel : public Commander {
  public:
-  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+  Status Execute(engine::Context &ctx, Server *srv, Connection *conn, std::string *output) override {
     std::vector<Slice> fields;
     for (size_t i = 2; i < args_.size(); i++) {
       fields.emplace_back(args_[i]);
@@ -98,7 +98,7 @@ class CommandHDel : public Commander {
 
     uint64_t ret = 0;
     redis::Hash hash_db(srv->storage, conn->GetNamespace());
-    engine::Context ctx(srv->storage);
+
     auto s = hash_db.Delete(ctx, args_[1], fields, &ret);
     if (!s.ok()) {
       return {Status::RedisExecErr, s.ToString()};
@@ -111,10 +111,10 @@ class CommandHDel : public Commander {
 
 class CommandHExists : public Commander {
  public:
-  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+  Status Execute(engine::Context &ctx, Server *srv, Connection *conn, std::string *output) override {
     redis::Hash hash_db(srv->storage, conn->GetNamespace());
     std::string value;
-    engine::Context ctx(srv->storage);
+
     auto s = hash_db.Get(ctx, args_[1], args_[2], &value);
     if (!s.ok() && !s.IsNotFound()) {
       return {Status::RedisExecErr, s.ToString()};
@@ -127,10 +127,10 @@ class CommandHExists : public Commander {
 
 class CommandHLen : public Commander {
  public:
-  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+  Status Execute(engine::Context &ctx, Server *srv, Connection *conn, std::string *output) override {
     uint64_t count = 0;
     redis::Hash hash_db(srv->storage, conn->GetNamespace());
-    engine::Context ctx(srv->storage);
+
     auto s = hash_db.Size(ctx, args_[1], &count);
     if (!s.ok() && !s.IsNotFound()) {
       return {Status::RedisExecErr, s.ToString()};
@@ -153,10 +153,10 @@ class CommandHIncrBy : public Commander {
     return Commander::Parse(args);
   }
 
-  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+  Status Execute(engine::Context &ctx, Server *srv, Connection *conn, std::string *output) override {
     int64_t ret = 0;
     redis::Hash hash_db(srv->storage, conn->GetNamespace());
-    engine::Context ctx(srv->storage);
+
     auto s = hash_db.IncrBy(ctx, args_[1], args_[2], increment_, &ret);
     if (!s.ok()) {
       return {Status::RedisExecErr, s.ToString()};
@@ -181,10 +181,10 @@ class CommandHIncrByFloat : public Commander {
     return Commander::Parse(args);
   }
 
-  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+  Status Execute(engine::Context &ctx, Server *srv, Connection *conn, std::string *output) override {
     double ret = 0;
     redis::Hash hash_db(srv->storage, conn->GetNamespace());
-    engine::Context ctx(srv->storage);
+
     auto s = hash_db.IncrByFloat(ctx, args_[1], args_[2], increment_, &ret);
     if (!s.ok()) {
       return {Status::RedisExecErr, s.ToString()};
@@ -200,7 +200,7 @@ class CommandHIncrByFloat : public Commander {
 
 class CommandHMGet : public Commander {
  public:
-  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+  Status Execute(engine::Context &ctx, Server *srv, Connection *conn, std::string *output) override {
     std::vector<Slice> fields;
     for (size_t i = 2; i < args_.size(); i++) {
       fields.emplace_back(args_[i]);
@@ -209,7 +209,7 @@ class CommandHMGet : public Commander {
     std::vector<std::string> values;
     std::vector<rocksdb::Status> statuses;
     redis::Hash hash_db(srv->storage, conn->GetNamespace());
-    engine::Context ctx(srv->storage);
+
     auto s = hash_db.MGet(ctx, args_[1], fields, &values, &statuses);
     if (!s.ok() && !s.IsNotFound()) {
       return {Status::RedisExecErr, s.ToString()};
@@ -237,10 +237,10 @@ class CommandHMSet : public Commander {
     return Commander::Parse(args);
   }
 
-  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+  Status Execute(engine::Context &ctx, Server *srv, Connection *conn, std::string *output) override {
     uint64_t ret = 0;
     redis::Hash hash_db(srv->storage, conn->GetNamespace());
-    engine::Context ctx(srv->storage);
+
     auto s = hash_db.MSet(ctx, args_[1], field_values_, false, &ret);
     if (!s.ok()) {
       return {Status::RedisExecErr, s.ToString()};
@@ -249,7 +249,7 @@ class CommandHMSet : public Commander {
     if (GetAttributes()->name == "hset") {
       *output = redis::Integer(ret);
     } else {
-      *output = redis::SimpleString("OK");
+      *output = redis::RESP_OK;
     }
     return Status::OK();
   }
@@ -260,10 +260,10 @@ class CommandHMSet : public Commander {
 
 class CommandHKeys : public Commander {
  public:
-  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+  Status Execute(engine::Context &ctx, Server *srv, Connection *conn, std::string *output) override {
     redis::Hash hash_db(srv->storage, conn->GetNamespace());
     std::vector<FieldValue> field_values;
-    engine::Context ctx(srv->storage);
+
     auto s = hash_db.GetAll(ctx, args_[1], &field_values, HashFetchType::kOnlyKey);
     if (!s.ok()) {
       return {Status::RedisExecErr, s.ToString()};
@@ -282,10 +282,10 @@ class CommandHKeys : public Commander {
 
 class CommandHVals : public Commander {
  public:
-  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+  Status Execute(engine::Context &ctx, Server *srv, Connection *conn, std::string *output) override {
     redis::Hash hash_db(srv->storage, conn->GetNamespace());
     std::vector<FieldValue> field_values;
-    engine::Context ctx(srv->storage);
+
     auto s = hash_db.GetAll(ctx, args_[1], &field_values, HashFetchType::kOnlyValue);
     if (!s.ok()) {
       return {Status::RedisExecErr, s.ToString()};
@@ -304,10 +304,10 @@ class CommandHVals : public Commander {
 
 class CommandHGetAll : public Commander {
  public:
-  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+  Status Execute(engine::Context &ctx, Server *srv, Connection *conn, std::string *output) override {
     redis::Hash hash_db(srv->storage, conn->GetNamespace());
     std::vector<FieldValue> field_values;
-    engine::Context ctx(srv->storage);
+
     auto s = hash_db.GetAll(ctx, args_[1], &field_values);
     if (!s.ok()) {
       return {Status::RedisExecErr, s.ToString()};
@@ -346,10 +346,10 @@ class CommandHRangeByLex : public Commander {
     }
   }
 
-  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+  Status Execute(engine::Context &ctx, Server *srv, Connection *conn, std::string *output) override {
     redis::Hash hash_db(srv->storage, conn->GetNamespace());
     std::vector<FieldValue> field_values;
-    engine::Context ctx(srv->storage);
+
     rocksdb::Status s = hash_db.RangeByLex(ctx, args_[1], spec_, &field_values);
     if (!s.ok()) {
       return {Status::RedisExecErr, s.ToString()};
@@ -371,12 +371,12 @@ class CommandHRangeByLex : public Commander {
 class CommandHScan : public CommandSubkeyScanBase {
  public:
   CommandHScan() = default;
-  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+  Status Execute(engine::Context &ctx, Server *srv, Connection *conn, std::string *output) override {
     redis::Hash hash_db(srv->storage, conn->GetNamespace());
     std::vector<std::string> fields;
     std::vector<std::string> values;
     auto key_name = srv->GetKeyNameFromCursor(cursor_, CursorType::kTypeHash);
-    engine::Context ctx(srv->storage);
+
     auto s = hash_db.Scan(ctx, key_, key_name, limit_, prefix_, &fields, &values);
     if (!s.ok() && !s.IsNotFound()) {
       return {Status::RedisExecErr, s.ToString()};
@@ -414,11 +414,10 @@ class CommandHRandField : public Commander {
     return Commander::Parse(args);
   }
 
-  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+  Status Execute(engine::Context &ctx, Server *srv, Connection *conn, std::string *output) override {
     redis::Hash hash_db(srv->storage, conn->GetNamespace());
     std::vector<FieldValue> field_values;
 
-    engine::Context ctx(srv->storage);
     auto s = hash_db.RandField(ctx, args_[1], command_count_, &field_values,
                                withvalues_ ? HashFetchType::kAll : HashFetchType::kOnlyKey);
     if (!s.ok() && !s.IsNotFound()) {
@@ -461,6 +460,6 @@ REDIS_REGISTER_COMMANDS(Hash, MakeCmdAttr<CommandHGet>("hget", 3, "read-only", 1
                         MakeCmdAttr<CommandHGetAll>("hgetall", 2, "read-only slow", 1, 1, 1),
                         MakeCmdAttr<CommandHScan>("hscan", -3, "read-only", 1, 1, 1),
                         MakeCmdAttr<CommandHRangeByLex>("hrangebylex", -4, "read-only", 1, 1, 1),
-                        MakeCmdAttr<CommandHRandField>("hrandfield", -2, "read-only", 1, 1, 1), )
+                        MakeCmdAttr<CommandHRandField>("hrandfield", -2, "read-only slow", 1, 1, 1), )
 
 }  // namespace redis
