@@ -39,10 +39,10 @@ class CommandSortedintAdd : public Commander {
     return Commander::Parse(args);
   }
 
-  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+  Status Execute(engine::Context &ctx, Server *srv, Connection *conn, std::string *output) override {
     redis::Sortedint sortedint_db(srv->storage, conn->GetNamespace());
     uint64_t ret = 0;
-    engine::Context ctx(srv->storage);
+
     auto s = sortedint_db.Add(ctx, args_[1], ids_, &ret);
     if (!s.ok()) {
       return {Status::RedisExecErr, s.ToString()};
@@ -70,10 +70,10 @@ class CommandSortedintRem : public Commander {
     return Commander::Parse(args);
   }
 
-  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+  Status Execute(engine::Context &ctx, Server *srv, Connection *conn, std::string *output) override {
     redis::Sortedint sortedint_db(srv->storage, conn->GetNamespace());
     uint64_t ret = 0;
-    engine::Context ctx(srv->storage);
+
     auto s = sortedint_db.Remove(ctx, args_[1], ids_, &ret);
     if (!s.ok()) {
       return {Status::RedisExecErr, s.ToString()};
@@ -89,10 +89,10 @@ class CommandSortedintRem : public Commander {
 
 class CommandSortedintCard : public Commander {
  public:
-  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+  Status Execute(engine::Context &ctx, Server *srv, Connection *conn, std::string *output) override {
     redis::Sortedint sortedint_db(srv->storage, conn->GetNamespace());
     uint64_t ret = 0;
-    engine::Context ctx(srv->storage);
+
     auto s = sortedint_db.Card(ctx, args_[1], &ret);
     if (!s.ok()) {
       return {Status::RedisExecErr, s.ToString()};
@@ -105,7 +105,7 @@ class CommandSortedintCard : public Commander {
 
 class CommandSortedintExists : public Commander {
  public:
-  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+  Status Execute(engine::Context &ctx, Server *srv, Connection *conn, std::string *output) override {
     redis::Sortedint sortedint_db(srv->storage, conn->GetNamespace());
     std::vector<uint64_t> ids;
     for (size_t i = 2; i < args_.size(); i++) {
@@ -118,7 +118,7 @@ class CommandSortedintExists : public Commander {
     }
 
     std::vector<int> exists;
-    engine::Context ctx(srv->storage);
+
     auto s = sortedint_db.MExist(ctx, args_[1], ids, &exists);
     if (!s.ok() && !s.IsNotFound()) {
       return {Status::RedisExecErr, s.ToString()};
@@ -166,10 +166,10 @@ class CommandSortedintRange : public Commander {
     return Commander::Parse(args);
   }
 
-  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+  Status Execute(engine::Context &ctx, Server *srv, Connection *conn, std::string *output) override {
     redis::Sortedint sortedint_db(srv->storage, conn->GetNamespace());
     std::vector<uint64_t> ids;
-    engine::Context ctx(srv->storage);
+
     auto s = sortedint_db.Range(ctx, args_[1], cursor_id_, offset_, limit_, reversed_, &ids);
     if (!s.ok()) {
       return {Status::RedisExecErr, s.ToString()};
@@ -228,11 +228,11 @@ class CommandSortedintRangeByValue : public Commander {
     return Commander::Parse(args);
   }
 
-  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+  Status Execute(engine::Context &ctx, Server *srv, Connection *conn, std::string *output) override {
     std::vector<uint64_t> ids;
     int size = 0;
     redis::Sortedint sortedint_db(srv->storage, conn->GetNamespace());
-    engine::Context ctx(srv->storage);
+
     auto s = sortedint_db.RangeByValue(ctx, args_[1], spec_, &ids, &size);
     if (!s.ok()) {
       return {Status::RedisExecErr, s.ToString()};
