@@ -108,7 +108,7 @@ Status Request::Tokenize(evbuffer *input) {
         }
 
         bulk_len_ = *parse_result;
-        if (bulk_len_ > PROTO_BULK_MAX_SIZE) {
+        if (bulk_len_ > srv_->GetConfig()->proto_max_bulk_len) {
           return {Status::NotOK, "Protocol error: invalid bulk length"};
         }
 
